# $Id: /local/Mango/trunk/lib/Mango.pm 313 2007-05-30T16:16:45.407876Z claco  $
package Mango;
use strict;
use warnings;

our $VERSION = '0.01000_05';

1;
__END__

=head1 NAME

Mango - An ecommerce solution using Catalyst, Handel and DBIx::Class

=head1 DESCRIPTION

Mango is an ecommerce solution built using Catalyst, Handel and DBIx::Class.
It is built using DBIx::Class for the database interaction and Handel for
the shopping cart/wishlist/order/checkout logic.

B<WARNING: This is an alpha release containing core modules only. No Catalyst
or web bits are included at this time.>

=head1 SOURCE

You can get the latest source from the Subversion repository located at
L<http://svn.mangoframework.com/CPAN/Mango/trunk>.

=head1 WEBSITE

Keep an eye on the website [L<http://mangoframework.com/blog/>] for the latest
news and releases.

=head1 CONTACT

If you need help or have any questions, stop by #mango@irc.perl.org or email
me at claco@chrislaco.com.

=head1 SEE ALSO

L<Handel>, L<DBIx::Class>

=head1 AUTHOR

    Christopher H. Laco
    CPAN ID: CLACO
    claco@chrislaco.com
    http://today.icantfocus.com/blog/
